import {
  setQuerybuilderNodes,
  store,
} from '@graphpolaris/shared/data-access/store';
import Graph from 'graphology';
import { Attributes } from 'graphology-types';

/** monospace fontsize table */
const widthPerFontsize = {
  6: 3.6167,
  7: 4.2167,
  10: 6.0167,
};

/** Adds a query builder pill to the graphology nodes object. */
export function addPill(
  id: string,
  attributes: Attributes,
  nodes: Graph
): boolean {
  const { type, name } = attributes;
  if (!type || !name) return false;
  let { x, y } = attributes;

  // Check if x and y are present, otherwise set them to 0
  if (!x) x = 0;
  if (!y) y = 0;

  // Get the width and height of a node
  const { w, h } = calcWidthHeightOfPill(attributes);

  // Add a node to the graphology object
  nodes.addNode(id, { ...attributes, x, y, w, h });

  // Set the new nodes in the query builder slice
  store.dispatch(setQuerybuilderNodes(nodes.export()));

  return true;
}

/** Calculates the width and height of a query builder pill.
 * DEPENDS ON STYLING, if styling changed, change this.
 */
function calcWidthHeightOfPill(attributes: Attributes): {
  w: number;
  h: number;
} {
  const { type, name } = attributes;

  let w = 0;
  let h = 0;
  switch (type) {
    case 'entity': {
      // calculate width and height of entity pill
      w = Math.min(name.length, 20) * widthPerFontsize[10]; // for fontsize 10px

      const widthOfPillWithoutText = 42.1164; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 20;
      break;
    }
    case 'relation': {
      // calculate width and height of relation pill
      w = Math.min(name.length, 20) * widthPerFontsize[10]; // for fontsize 10px

      const widthOfPillWithoutText = 56.0666; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 20;
      break;
    }
    case 'attribute': {
      // calculate width and height of relation pill
      const pixelsPerChar = widthPerFontsize[6]; // for fontsize 10px
      w = name.length * pixelsPerChar;

      const { datatype, operator } = attributes;
      let value = attributes['value'];
      if (!datatype || !operator) return { w: 0, h: 0 };
      if (!value) value = '?';

      // Add width of operator
      w += operator.length * widthPerFontsize[7];
      // use a max of 10, because max-width is set to 10ch;
      w += Math.min(value.length, 10) * widthPerFontsize[6];

      const widthOfPillWithoutText = 25.6666; // WARNING: depends on styling
      w += widthOfPillWithoutText;
      h = 12;
      break;
    }
  }

  return { w, h };
}
